@@ -15,7 +15,9 @@
 #include <gvc/gvc.h>
 #include <xdot/xdot.h>
 #include <cgraph/agxbuf.h>
+#include <cgraph/alloc.h>
 #include <cgraph/exit.h>
+#include <cgraph/startswith.h>
 #include <cgraph/strcasecmp.h>
 #include <stdbool.h>
 #include <stddef.h>
@@ -177,19 +179,21 @@ static void use_library(GVC_t *gvc, const char *name)
     gvc->common.lib = Lib;
 }
 
-static void global_def(agxbuf* xb, char *dcl, int kind) {
+static void global_def(char *dcl, int kind) {
     char *p;
     char *rhs = "true";
+    agxbuf xb = {0};
 
     attrsym_t *sym;
     if ((p = strchr(dcl, '='))) {
-        agxbput_n(xb, dcl, (size_t)(p - dcl));
+        agxbput_n(&xb, dcl, (size_t)(p - dcl));
         rhs = p+1;
     }
     else
-	agxbput (xb, dcl);
-    sym = agattr(NULL, kind, agxbuse (xb), rhs);
+	agxbput(&xb, dcl);
+    sym = agattr(NULL, kind, agxbuse(&xb), rhs);
     sym->fixed = 1;
+    agxbfree(&xb);
 }
 
 static int gvg_init(GVC_t *gvc, graph_t *g, char *fn, int gidx)
@@ -227,15 +231,18 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
     char c, *rest, *layout;
     const char *val;
     int i, v, nfiles;
-    char buf[SMALLBUF];
-    agxbuf xb;
     int Kflag = 0;
 
     /* establish if we are running in a CGI environment */
     HTTPServerEnVar = getenv("SERVER_NAME");
 
-    /* establish Gvfilepath, if any */
-    Gvfilepath = getenv("GV_FILE_PATH");
+    // test `$GV_FILE_PATH`, a legacy knob, is not set
+    if (getenv("GV_FILE_PATH") != NULL) {
+        fprintf(stderr, "$GV_FILE_PATH environment variable set; exiting\n"
+                        "\n"
+                        "This sandboxing mechanism is no longer supported\n");
+        graphviz_exit(EXIT_FAILURE);
+    }
 
     gvc->common.cmdname = dotneato_basename(argv[0]);
     if (gvc->common.verbose) {
@@ -262,14 +269,23 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 	    nfiles++;
     gvc->input_filenames = N_NEW(nfiles + 1, char *);
     nfiles = 0;
-    agxbinit(&xb, SMALLBUF, buf);
     for (i = 1; i < argc; i++) {
-	if (argv[i] && argv[i][0] == '-') {
+	if (argv[i] &&
+	    (startswith(argv[i], "-V") || strcmp(argv[i], "--version") == 0)) {
+	    fprintf(stderr, "%s - %s version %s (%s)\n",
+	            gvc->common.cmdname, gvc->common.info[0],
+	            gvc->common.info[1], gvc->common.info[2]);
+	    if (GvExitOnUsage) graphviz_exit(0);
+	    return 1;
+	} else if (argv[i] &&
+	    (startswith(argv[i], "-?") || strcmp(argv[i], "--help") == 0)) {
+	    return dotneato_usage(0);
+	} else if (argv[i] && argv[i][0] == '-') {
 	    rest = &argv[i][2];
 	    switch (c = argv[i][1]) {
 	    case 'G':
 		if (*rest)
-		    global_def(&xb, rest, AGRAPH);
+		    global_def(rest, AGRAPH);
 		else {
 		    fprintf(stderr, "Missing argument for -G flag\n");
 		    return (dotneato_usage(1));
@@ -277,7 +293,7 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 		break;
 	    case 'N':
 		if (*rest)
-		    global_def(&xb, rest, AGNODE);
+		    global_def(rest, AGNODE);
 		else {
 		    fprintf(stderr, "Missing argument for -N flag\n");
 		    return (dotneato_usage(1));
@@ -285,7 +301,7 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 		break;
 	    case 'E':
 		if (*rest)
-		    global_def(&xb, rest, AGEDGE);
+		    global_def(rest, AGEDGE);
 		else {
 		    fprintf(stderr, "Missing argument for -E flag\n");
 		    return (dotneato_usage(1));
@@ -344,13 +360,6 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 	    case 'P':
 		P_graph = gvplugin_graph(gvc);
 		break;
-	    case 'V':
-		fprintf(stderr, "%s - %s version %s (%s)\n",
-			gvc->common.cmdname, gvc->common.info[0], 
-			gvc->common.info[1], gvc->common.info[2]);
-		if (GvExitOnUsage) graphviz_exit(0);
-		return (1);
-		break;
 	    case 'l':
 		val = getFlagOpt(argc, argv, &i);
 		if (!val) {
@@ -402,9 +411,6 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 	    case 'y':
 		Y_invert = TRUE;
 		break;
-	    case '?':
-		return (dotneato_usage(0));
-		break;
 	    default:
 		agerr(AGERR, "%s: option -%c unrecognized\n\n", gvc->common.cmdname,
 			c);
@@ -413,7 +419,6 @@ int dotneato_args_initialize(GVC_t * gvc, int argc, char **argv)
 	} else if (argv[i])
 	    gvc->input_filenames[nfiles++] = argv[i];
     }
-    agxbfree (&xb);
 
     /* if no -K, use cmd name to set layout type */
     if (!Kflag) {
@@ -628,13 +633,14 @@ void graph_init(graph_t * g, bool use_rankdir)
     static char *fontnamenames[] = {"gd","ps","svg", NULL};
     static int fontnamecodes[] = {NATIVEFONTS,PSFONTS,SVGFONTS,-1};
     int rankdir;
-    GD_drawing(g) = NEW(layout_t);
+    GD_drawing(g) = gv_alloc(sizeof(layout_t));
 
     /* reparseable input */
     if ((p = agget(g, "postaction"))) {   /* requires a graph wrapper for yyparse */
-        char *buf = gmalloc(strlen("digraph {  }") + strlen(p) + 1);
-        sprintf(buf,"%s { %s }",agisdirected(g)?"digraph":"graph",p);
-        agmemconcat(g, buf);
+        agxbuf buf = {0};
+        agxbprint(&buf, "%s { %s }", agisdirected(g) ? "digraph" : "graph", p);
+        agmemconcat(g, agxbuse(&buf));
+        agxbfree(&buf);
     }
 
     /* set this up fairly early in case any string sizes are needed */
@@ -644,12 +650,9 @@ void graph_init(graph_t * g, bool use_rankdir)
 #ifdef HAVE_SETENV
 	setenv("GDFONTPATH", p, 1);
 #else
-	static char *buf = 0;
-
-	buf = grealloc(buf, strlen("GDFONTPATH=") + strlen(p) + 1);
-	strcpy(buf, "GDFONTPATH=");
-	strcat(buf, p);
-	putenv(buf);
+	static agxbuf buf;
+	agxbprint(&buf, "GDFONTPATH=%s", p);
+	putenv(agxbuse(&buf));
 #endif
     }
 
@@ -657,8 +660,6 @@ void graph_init(graph_t * g, bool use_rankdir)
 
     if (!HTTPServerEnVar) {
 	Gvimagepath = agget (g, "imagepath");
-	if (!Gvimagepath)
-	    Gvimagepath = Gvfilepath;
     }
 
     GD_drawing(g)->quantum =
@@ -785,7 +786,6 @@ void graph_init(graph_t * g, bool use_rankdir)
     E_label = agfindedgeattr(g, "label");
     E_xlabel = agfindedgeattr(g, "xlabel");
     E_label_float = agfindedgeattr(g, "labelfloat");
-    /* vladimir */
     E_dir = agfindedgeattr(g, "dir");
     E_arrowhead = agfindedgeattr(g, "arrowhead");
     E_arrowtail = agfindedgeattr(g, "arrowtail");
@@ -796,7 +796,6 @@ void graph_init(graph_t * g, bool use_rankdir)
     E_labelfontcolor = agfindedgeattr(g, "labelfontcolor");
     E_labeldistance = agfindedgeattr(g, "labeldistance");
     E_labelangle = agfindedgeattr(g, "labelangle");
-    /* end vladimir */
     E_minlen = agfindedgeattr(g, "minlen");
     E_showboxes = agfindedgeattr(g, "showboxes");
     E_style = agfindedgeattr(g, "style");
@@ -820,7 +819,7 @@ void graph_init(graph_t * g, bool use_rankdir)
 void graph_cleanup(graph_t *g)
 {
     if (GD_drawing(g) && GD_drawing(g)->xdots)
-	freeXDot ((xdot*)GD_drawing(g)->xdots);
+	freeXDot(GD_drawing(g)->xdots);
     if (GD_drawing(g))
 	free (GD_drawing(g)->id);
     free(GD_drawing(g));
