@@ -15,6 +15,7 @@
 #include <math.h>
 #include <string.h>
 #include <assert.h>
+#include <cgraph/exit.h>
 /* Applications that do not use the common library can define STANDALONE
  * to get definitions/definitions that are normally provided there.
  * In particular, note that Verbose is declared but undefined.
@@ -22,12 +23,13 @@
 #ifndef STANDALONE
 #include "cgraph.h"
 #include "globals.h"
-#include "logic.h"
 #include "arith.h"
 #include <common/memory.h>
 #endif  /* STANDALONE */
 
-#define real double
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 #define set_flag(a, flag) ((a)=((a)|(flag)))
 #define test_flag(a, flag) ((a)&(flag))
@@ -57,8 +59,11 @@
 
 #define POINTS(inch) 72*(inch)
 
-typedef unsigned int boolean;
+#ifdef GVDLL
+__declspec(dllimport) extern unsigned char Verbose;
+#else
 extern unsigned char Verbose;
+#endif
 
 #else  /* STANDALONE */
 #define CALLOC gcalloc
@@ -66,32 +71,24 @@ extern unsigned char Verbose;
 #define REALLOC grealloc
 #endif    /* STANDALONE */
 
-#define FREE free
-
-#ifndef DEBUG
-#ifndef NDEBUG
-#define NDEBUG /* switch off assert*/
-#endif
-#endif
-
 #ifdef DEBUG
 extern double _statistics[10];
 #endif
 
 
 extern int irand(int n);
-extern real drand(void);
+extern double drand(void);
 extern int *random_permutation(int n);/* random permutation of 0 to n-1 */
 
 
-real* vector_subtract_to(int n, real *x, real *y);/* y = x-y */
+double* vector_subtract_to(int n, double *x, double *y);/* y = x-y */
 
-real vector_product(int n, real *x, real *y);
+double vector_product(int n, double *x, double *y);
 
-real* vector_saxpy(int n, real *x, real *y, real beta); /* y = x+beta*y */
+double* vector_saxpy(int n, double *x, double *y, double beta); /* y = x+beta*y */
 
 
-real* vector_saxpy2(int n, real *x, real *y, real beta);/* x = x+beta*y */
+double* vector_saxpy2(int n, double *x, double *y, double beta);/* x = x+beta*y */
 
 /* take m elements v[p[i]]],i=1,...,m and oput in u. u will be assigned if *u = NULL */
 void vector_float_take(int n, float *v, int m, int *p, float **u);
@@ -99,10 +96,10 @@ void vector_float_take(int n, float *v, int m, int *p, float **u);
 /* give the position of the smallest, second smallest etc in vector v.
    results in p. If *p == NULL, p is assigned.
 */
-void vector_ordering(int n, real *v, int **p);
+void vector_ordering(int n, double *v, int **p);
 void vector_sort_int(int n, int *v);
 
-void vector_print(char *s, int n, real *x);
+void vector_print(char *s, int n, double *x);
 
 #define MACHINEACC 1.0e-16
 #define SQRT_MACHINEACC 1.0e-8
@@ -112,11 +109,15 @@ void vector_print(char *s, int n, real *x);
 enum {UNMATCHED = -1};
 
 
-real distance(real *x, int dim, int i, int j);
-real distance_cropped(real *x, int dim, int i, int j);
+double distance(double *x, int dim, int i, int j);
+double distance_cropped(double *x, int dim, int i, int j);
 
-real point_distance(real *p1, real *p2, int dim);
+double point_distance(double *p1, double *p2, int dim);
 
 char *strip_dir(char *s);
 
-void scale_to_box(real xmin, real ymin, real xmax, real ymax, int n, int dim, real *x);
+void scale_to_box(double xmin, double ymin, double xmax, double ymax, int n, int dim, double *x);
+
+#ifdef __cplusplus
+}
+#endif
